library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;
use sources.pack_memory.all;


-- Fichier Memory_with_cache.vhdl


entity Memory_with_cache is
	port(
	-- entrees
		data_in  : in t_mot;  
		address  : in t_address;
		cs	 	 : in std_logic;
		rw	 	 : in std_logic;
		reset	 : in std_logic;

	-- nouvelles entres
		clock 	 : in std_logic;
	-- sorties
		data_out : out t_mot;
		ready 	 : out std_logic;
	-- sorties qui permet d'afficher les mmoires
		sasa	 : out t_cache);
end Memory_with_cache;


architecture cache_LRU_arch of Memory_with_cache is

-- on rappelle la signature de l'entit main_memory
component main_memory is
generic	(
	-- paramtres de de la mmoire principale --
	N : integer range 1 to 64; 	-- taille de l'adresse => taille de la mmoire 2**N mots
	TM : integer range 0 to 8; 	-- taille d'un mot : 2**TM
	L : integer range 0 to 63	-- latence de la mmoire
);
port (
	reset : in std_logic; -- rinitialisation (niveau bas)
	clock : in std_logic; -- horloge (front montant)
	data_in : in std_logic_vector(1 to TM);		-- donne  crire
	data_out : out std_logic_vector(1 to TM);	-- donne  lire
	address : in std_logic_vector(N-1 downto 0); -- adresse de la donne  crire ou lire
	rw : in std_logic; -- 1 : read / 0 : write
	cs : in std_logic -- activation du boitier (niveau bas)
);
end component;





-- on dclare les signaux utiles
signal mc : t_cache;
signal tmp_rw, tmp_cs : std_logic;
signal tmp_data_in, tmp_data_out : std_logic_vector(1 to 8);
signal tmp_address : std_logic_vector(10 downto 0);
signal etat : t_etat;
signal cptload : t_state;
	

begin

-- une instance du composant mmoire de 2Ko avec temps de latence de 10 cycles
u_memory : main_memory
generic map (N => 11, TM => 8, L => Latence)
port map (
	clock => clock, reset => reset, data_in => tmp_data_in, data_out => tmp_data_out, 
		address => tmp_address, rw => tmp_rw, cs => tmp_cs);
	
	
	process (clock,reset) is
	
		variable etiquette_cur : std_logic_vector(5 downto 0);
		variable num_ensemble_cur : integer;
		variable ensemble_cur : t_ensemble ;
		variable off_set_cur : integer;
		variable defaut : boolean;
		variable bl : t_bloc;
		variable pos_LFU : integer;
		variable i : integer; 
		variable continuer : boolean;
		variable cptLatence : integer;	
	begin
		
		if reset='0' then
									
			-- on initialise la memoire cache
			-- Pour chaque ensemble
			for i in 0 to 7 loop
				-- Pour chaque ligne
				for j in 0 to 15 loop
					mc(i)(j).bit_validite <= '0';
					mc(i)(j).tag <= j;
					mc(i)(j).maj <= '0';
				end loop;
			end loop;
			
			-- On est dans l'tat DECODE
			etat <= DECODE;
			-- La mmoire est prete
			ready <= '0';
		
		elsif clock'event and clock='1' then
			
			-- les tats de la mmoire
-- DECODE : mmoire prte : on dcode la prochaine instruction
-- SEARCH : dcodage termin, recherche de la donne dans le cache
-- LOAD   : chargement de la donne en mmoire cache
-- EXEC   : execution de l'instruction

			case etat is
			
				when DECODE => 
				
					if(cs='0') then	
						-- La mmoire n'est plus accessible
						ready <= '1';
					
						etiquette_cur    	:= address(10 downto 5);
						num_ensemble_cur 	:= conv_integer(address(4 downto 2));
						ensemble_cur		:= mc(num_ensemble_cur);
						off_set_cur	  	:= conv_integer(address(1 downto 0));
					
						-- On passe dans l'etat de recherche
						etat     <= SEARCH;
					 end if;
					 
				when SEARCH =>
				
					if(cs='1') then	
						defaut := true; 
						
						-- on parcours l'ensemble
						for i in 0 to 15 loop
							if (ensemble_cur(i).bit_validite = '1') then
								if ( ensemble_cur(i).etiquette = etiquette_cur ) then
									-- MAJ du tag concern
									mc(num_ensemble_cur)(i).tag <= mc(num_ensemble_cur)(i).tag + 1;
									-- Il n'y a pas de dfaut de cache
									defaut := false;
								end if;
							end if;
						end loop;
						if defaut then
						-- s'il y a defaut de cache
							-- On initialise cptLatence  0
							cptLatence := 0;
							-- On initialise le cpt  la bonne valeur
							cptload <= LECTURE1;
							-- On passe dans l'etat de chargement
							etat <= LOAD;
							
						else
						-- Sinon
							-- On passe dans l'etat d'execution
							etat <= EXEC;
						end if;
					end if;
					
				when LOAD   =>
					
					if(cs='1') then	
						case cptload is
						
							when LECTURE1 =>
							
								if(cptLatence = 0)then
								-- Mode lecture
								tmp_rw <= '1';
								-- CS actif
								tmp_cs <= '0';
								-- On construit l'address
								tmp_address <= address(10 downto 2) & conv_std_logic_vector( 0 , 2 );	
								-- On recupre ce qu'on a lu
								
								end if;
								if(cptLatence < Latence)then
									cptLatence := cptLatence + 1 ;								
								else	
									cptLatence := 0 ;								
									bl(0) := tmp_data_out;
									cptload <= LECTURE2 ;
								end if;					
								
							when LECTURE2 =>
															
								if(cptLatence = 0)then
								-- On construit l'address
								tmp_address <= address(10 downto 2) & conv_std_logic_vector( 1 , 2 );	
								-- On recupre ce qu'on a lu
								
								end if;
								if(cptLatence < Latence)then
									cptLatence := cptLatence + 1 ;								
								else	
									cptLatence := 0 ;								
									bl(1) := tmp_data_out;
									cptload <= LECTURE3 ;
								end if;	

								
							when LECTURE3 =>
								
								
								if(cptLatence = 0)then
								-- On construit l'address
								tmp_address <= address(10 downto 2) & conv_std_logic_vector( 2 , 2 );	
								-- On recupre ce qu'on a lu
								end if;
								if(cptLatence < Latence)then
									cptLatence := cptLatence + 1 ;								
								else	
									cptLatence := 0 ;								
									bl(2) := tmp_data_out;
									cptload <= LECTURE4 ;
								end if;								
								
							when LECTURE4 =>
							
								
								if(cptLatence = 0)then
								-- On construit l'address
								tmp_address <= address(10 downto 2) & conv_std_logic_vector( 3 , 2 );	
								-- On recupre ce qu'on a lu
								end if;
								if(cptLatence < Latence)then
									cptLatence := cptLatence + 1 ;								
								else	
									cptLatence := 0 ;								
									bl(3) := tmp_data_out;
									cptload <= RECHERCHE ;
								end if;
								
							
							when RECHERCHE =>
								
								-- Desactivation du CS
								tmp_cs <= '1';
								
								-- On recherche la position ou sera ajoute les donnes
								pos_LFU := 0;
								i := 0;
								continuer := true;
								while i <= 15 and continuer loop
									-- Si une ligne n'est pas affecte => on stock a cet endroit
									if mc(num_ensemble_cur)(i).bit_validite = '0' then
										mc(num_ensemble_cur)(i).bloc <= bl ;
										mc(num_ensemble_cur)(i).etiquette <= address(10 downto 5) ;
										mc(num_ensemble_cur)(i).tag <= 1 ;
										mc(num_ensemble_cur)(i).bit_validite <= '1' ;
										mc(num_ensemble_cur)(i).maj <= '0' ;
										pos_LFU := -1 ;
										continuer := false ;
										
										-- On passe dans l'tat EXEC
										etat <= EXEC;			
										
									-- Sinon on regarde le tag courant
									else
										-- Si le tag courant est plus petit que l'actuel min, on change le min
										if (mc(num_ensemble_cur)(i).tag < mc(num_ensemble_cur)(pos_LFU).tag) then
											pos_LFU := i;
										end if;
									end if;
									i := i + 1;
								end loop;
								
								
								-- S'il n'y avait aucune ligne de 'libre'
								if pos_LFU /= -1 then
									-- Si on a modifi le contenu du bloc, on le recharge en mmoire principale avant de l'craser
									if mc(num_ensemble_cur)(pos_LFU).maj /= '1' then
											
										-- On charge directement la nouvelle donne
										mc(num_ensemble_cur)(pos_LFU).bloc <= bl ;
										mc(num_ensemble_cur)(pos_LFU).etiquette <= address(10 downto 5) ;
										mc(num_ensemble_cur)(pos_LFU).tag <= 1 ;
										mc(num_ensemble_cur)(pos_LFU).maj <= '0' ;
											
										-- On passe dans l'tat EXEC
										etat <= EXEC;
										
											
									else
										-- et on passe dans l'tat 5
										cptload <= ECRITURE1;
										
									end if;
								end if; 
								
								
							when ECRITURE1 =>
								if(cptLatence = 0)then
								-- On recharge en memoire principale le premier mot
								tmp_rw <= '0';-- mode criture
								tmp_cs <= '0';-- activation du boitier
								tmp_data_in <= mc(num_ensemble_cur)(pos_LFU).bloc(0);
								tmp_address <= address(10 downto 2) & conv_std_logic_vector( 0 , 2 );				
								end if;
								if(cptLatence < Latence)then
									cptLatence := cptLatence + 1 ;								
								else	
									cptLatence := 0 ;								
									-- et on passe dans l'tat 6
									cptload <= ECRITURE2;
								end if;
								
								
								
							when ECRITURE2 =>
							
								if(cptLatence = 0)then
								-- On recharge en memoire principale le deuxieme mot
								tmp_data_in <= mc(num_ensemble_cur)(pos_LFU).bloc(1);
								tmp_address <= address(10 downto 2) & conv_std_logic_vector( 1 , 2 );
								end if;
								if(cptLatence < Latence)then
									cptLatence := cptLatence + 1 ;								
								else	
									cptLatence := 0 ;								
									-- et on passe dans l'tat 7
									cptload <= ECRITURE3;
								end if;
								
								
								
							when ECRITURE3 =>	
								if(cptLatence = 0)then
								-- On recharge en memoire principale le troisieme mot
									tmp_data_in <= mc(num_ensemble_cur)(pos_LFU).bloc(2);
									tmp_address <= address(10 downto 2) & conv_std_logic_vector( 2 , 2 );
								end if;	
									if(cptLatence < Latence)then
										cptLatence := cptLatence + 1 ;								
									else	
										cptLatence := 0 ;								
										-- et on passe dans l'tat 8
										cptload <= ECRITURE4;
									end if;
									
							
							when ECRITURE4 =>
									if(cptLatence = 0)then
									-- On recharge en memoire principale le dernier mot
									tmp_data_in <= mc(num_ensemble_cur)(pos_LFU).bloc(3);
									tmp_address <= address(10 downto 2) & conv_std_logic_vector( 3 , 2 );
									end if;
									if(cptLatence < Latence)then
										cptLatence := cptLatence + 1 ;								
									else	
										cptLatence := 0 ;								
										-- et on passe dans l'tat 9
										cptload <= FINAL;
									end if;
							when FINAL =>
							
								-- Desactivation du boitier
								tmp_cs <= '1';
								
								-- On charge directement la nouvelle donne
								mc(num_ensemble_cur)(pos_LFU).bloc <= bl ;
								mc(num_ensemble_cur)(pos_LFU).etiquette <= address(10 downto 5) ;
								mc(num_ensemble_cur)(pos_LFU).tag <= 1 ;
								mc(num_ensemble_cur)(pos_LFU).maj <= '0' ;
											
								-- On passe dans l'tat EXEC
								etat <= EXEC;	
							
						end case;
					end if;	
					
				when EXEC   =>
				
					if(cs='1') then	
						if rw='0' then
							-- criture			
							for i in 0 to 15 loop
								if (mc(num_ensemble_cur)(i).bit_validite = '1') then
									if ( mc(num_ensemble_cur)(i).etiquette = etiquette_cur ) then
										-- on ecrit la nouvelle donne
										mc(num_ensemble_cur)(i).bloc(off_set_cur) <= data_in;
										-- on prcise qu'il y a eu une M.A.J.
										mc(num_ensemble_cur)(i).maj <= '1';
									end if;
								end if;
							end loop;
						else
							-- lecture	
							for i in 0 to 15 loop
								if (mc(num_ensemble_cur)(i).bit_validite = '1') then
									if ( mc(num_ensemble_cur)(i).etiquette = etiquette_cur ) then
										-- on rcupre la donne en question
										data_out <= mc(num_ensemble_cur)(i).bloc(off_set_cur);
									end if;
								end if;
							end loop;
						end if;
					
						-- La mmoire est a nouveau disponible
						ready <= '0';

						
						-- On repasse dans l'etat decode
						etat <= DECODE;
						
					end if;
					
			end case;	
			
		end if;
			
			sasa <= mc;
	end process;

end cache_LRU_arch;		



























