library ieee;
use ieee.Std_logic_1164.all;
use sources.pack_Memory.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

package charger_en_memoire_cache is
	procedure charger_en_memoire_cache_LFU (
		signal mc : inout t_cache ;
		signal mm : inout main_memory;
		signal address  : in std_logic_vector(10 downto 0)
	);
	
	procedure charger_en_memoire_cache_LRU ( 
		signal mc : inout t_cache ;
		signal mm : inout main_memory;
		signal address  : in std_logic_vector(10 downto 0);
		variable b : in t_bloc
	);
	
end;

package body charger_en_memoire_cache is

	procedure charger_en_memoire_cache_LFU ( 
				signal mc : inout t_cache ;
				signal mm : inout t_main_memory;
				signal address  : in std_logic_vector(10 downto 0)
				) is
		variable index : integer ;
		variable pos_LFU : integer;
		variable pos_mm : integer;
		variable i	: integer;
		variable continuer : boolean;
	begin
		index := conv_integer( address( 4 downto 2 ) );
		
		pos_LFU := 0;
		
		i := 0;
		continuer := true;
		while i <= 15 and continuer loop
			-- Si un ligne n'est pas affecte => on stock a cet endroit
			if mc(index)(i).bit_validite = '0' then
				mc(index)(i).bloc <= b;
				mc(index)(i).etiquette <= address(10 downto 5);
				mc(index)(i).tag <= 1;
				mc(index)(i).bit_validite <= '1';
				mc(index)(i).maj <= '0';
				pos_LFU := -1;
				continuer := false;			
			-- Sinon on regarde le tag courant
			else
				-- Si le tag courant est plus petit que l'actuel min, on change le min
				if (mc(index)(i).tag < mc(index)(pos_LFU).tag) then
					pos_LFU := i;
				end if;
			end if;
			
			i := i + 1;
		end loop;
		
		-- S'il n'y avait aucune ligne de 'libre'
		if pos_LFU /= -1 then
			-- Si on a modifi le contenu du bloc, on le recharge en mmoire principale avant de l'craser
			if mc(index)(pos_LFU).maj = '1' then
				------------------------------
				-- vrifier la concatnation--
				------------------------------
				pos_mm := conv_integer( mc(index)(pos_LFU).etiquette & address(4 downto 2) );	
				mm(pos_mm) <= mc(index)(pos_LFU).bloc;
			end if;
			-- On charge ensuite la nouvelle donne
			mc(index)(pos_LFU).bloc <= b;
			mc(index)(pos_LFU).etiquette <= address(10 downto 5);
			mc(index)(pos_LFU).tag <= 1;
			mc(index)(pos_LFU).maj <= '0';
		end if; 
		wait for 0 ns;
	end charger_en_memoire_cache_LFU;
	
	procedure charger_en_memoire_cache_LRU (
		signal mc : inout t_cache ;
		signal mm : inout t_main_memory;
		signal address  : in std_logic_vector(10 downto 0)
	) is
	variable index : integer ;
	variable pos_LRU : integer;
	variable pos_mm : integer;
	variable i : integer;
	variable continuer : boolean;
	
	begin
	
		index := conv_integer( address( 4 downto 2 ) );
	    
		pos_LRU := 0;
	    
	    	i := 0;
	    	continuer := true;
		while i<=15 and continuer loop
			-- Si une ligne n'est pas affecte => on stock a cet endroit
			if mc(index)(i).bit_validite = '0' then
				mc(index)(i).bloc <= b;
				mc(index)(i).etiquette <= address(10 downto 5);
				mc(index)(i).tag <= 0;
				mc(index)(i).bit_validite <= '1';
				mc(index)(i).maj <= '0';
				pos_LRU := -1;
				continuer := false;			
			-- Sinon on regarde le tag courant
			else
				-- Si le tag courant est plus petit que l'actuel min, on change le min
				if (mc(index)(i).tag > mc(index)(pos_LRU).tag) then
					pos_LRU := i;
				end if;
			end if;
			i := i + 1;
		end loop;
	    
		-- S'il n'y avait aucune ligne de 'libre'
		if pos_LRU /= -1 then
			-- Si on a modifi le contenu du bloc, on le recharge en mmoire principale avant de l'craser
			if mc(index)(pos_LRU).maj = '1' then
				------------------------------
				-- vrifier la concatnation--
				------------------------------
				pos_mm := conv_integer( mc(index)(pos_LRU).etiquette & address(4 downto 2) );	
				mm(pos_mm) <= mc(index)(pos_LRU).bloc;
			end if;
			-- On charge ensuite la nouvelle donne
			mc(index)(pos_LRU).bloc <= b;
			mc(index)(pos_LRU).etiquette <= address(10 downto 5);
			mc(index)(pos_LRU).tag <= 0;
			mc(index)(pos_LRU).maj <= '0';
		    
		end if; 
		wait for 0 ns;
		-- Mise  jour de tous les tags
		for j in 0 to 15 loop
			if ( mc(index)(j).bit_validite = '1' ) then
				mc(index)(j).tag <= mc(index)(j).tag + 1;
			end if;
		end loop;

		
	
	end charger_en_memoire_cache_LRU;


	
	
	
end charger_en_memoire_cache;
















