----------------------------------------------------------------
-- Test Bench pour le composant main_memory                   --
-- auteur : Matthias Colin                                    --
-- date   : 03/05/2010                                        --
----------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

entity tb_main_memory is
end tb_main_memory;

architecture tb_main_memory_arch of tb_main_memory is

component main_memory is
generic	(
	-- paramtres de de la mmoire principale --
	N : integer range 1 to 64; 	-- taille de l'adresse => taille de la mmoire 2**N mots
	TM : integer range 0 to 8; 	-- taille d'un mot : 2**TM
	L : integer range 0 to 63	-- latence de la mmoire
);
port (
	reset : in std_logic; -- rinitialisation (niveau bas)
	clock : in std_logic; -- horloge (front montant)
	data_in : in std_logic_vector(1 to TM);		-- donne  crire
	data_out : out std_logic_vector(1 to TM);	-- donne  lire
	address : in std_logic_vector(N-1 downto 0); -- adresse de la donne  crire ou lire
	rw : in std_logic; -- 1 : read / 0 : write
	cs : in std_logic -- activation du boitier (niveau bas)
);
end component;

signal tb_clock, tb_reset, tb_rw, tb_cs : std_logic;
signal tb_data_in, tb_data_out : std_logic_vector(1 to 8);
signal tb_address : std_logic_vector(10 downto 0);


begin

-- une instance du composant mmoire de 2Kio avec temps de latence de 20 cycles
u_memory : main_memory
 generic map (N => 11, TM => 8, L => 20)
 port map (
	clock => tb_clock, reset => tb_reset, data_in => tb_data_in, data_out => tb_data_out, 
		address => tb_address, rw => tb_rw, cs => tb_cs);

-- gnration d'un signal d'horloge de priode 20 ns
process
begin
	tb_clock <= '0';
	wait for 10 ns;
	tb_clock <= '1';
	wait for 10 ns;
end process;

-- test du composant mmoire
process
begin
	-- initialisation
	tb_reset <= '0';
	tb_cs <= '1';
	tb_address <= (others => 'X');
	tb_data_in <= (others => 'X');
	tb_rw <= 'X';
	wait for 20 ns;
	tb_reset <= '1';
	
	wait for 11 ns;
	-- 1er top aprs initialisation
	
	-- test 1 : criture en rafale de la mmoire
	for i in 0 to 25 loop
		tb_cs <= '0';
		tb_rw <= '0';
		tb_address <= conv_std_logic_vector(i,11);
		tb_data_in <= conv_std_logic_vector(i,8); -- on crit les 8 bits de pds faible de l'@
		wait for 20 ns;
	end loop;
	
	-- 20 cycles de repos (latence)
	tb_cs <= '1';
	wait for 400 ns;
	
	-- test 2 : lecture en rafale de la mmoire
	-- rsultat  lire dans tb_data_out
	for i in 0 to 25 loop
		tb_cs <= '0';
		tb_rw <= '1';
		tb_address <= conv_std_logic_vector(i,11);
		wait for 20 ns;
	end loop;
	
	tb_cs <= '1';
	wait for 400 ns;

	
	-- test 3 : criture puis lecture d'une mme case mmoire
	for i in 30 to 60 loop	
		-- 1 - ECRITURE
		-- adresse
		tb_address <= conv_std_logic_vector(i, 11);
		-- donne
		tb_data_in <= conv_std_logic_vector(i, 8);
		-- ordre écriture
		tb_cs <= '0';
		tb_rw <= '0';
		wait for 20 ns;
		tb_cs <= '1';

		-- 2 - REPOS
		-- 20 cycle de repos (latence)
		tb_cs <= '1';
		wait for 400 ns;
			
		-- 3 - LECTURE
		-- mme adresse (i sur 11 bits)
		-- ordre lecture :
		tb_cs <= '0';
		tb_rw <= '1';
		wait for 20 ns;
		tb_cs <= '1';
		
		-- 4 - REPOS
		-- 20 cycle de repos (latence)
		tb_cs <= '1';
		wait for 400 ns;
	end loop;
	
	wait for 400 ns;
end process;

end tb_main_memory_arch;










